namespace Micro.Interfaces.SymTab
{
    public interface ISymTab
    {
        /// <summary>
        /// Open a new top scope
        /// </summary>
        void OpenScope();

        /// <summary>
        /// Close the current top scope
        /// </summary>
        void CloseScope();

        /// <summary>
        /// Create a new symbol with the given parameters and insert
        /// it into the top scope
        /// </summary>
        /// <param name="kind"></param>
        /// <param name="name"></param>
        /// <param name="type"></param>
        /// <returns>
        /// The newly created symbol
        /// </returns>
        ISymbol Insert(SymbolKind kind, string name, IStruct type);

        /// <summary>
        /// Create a new symbol with the given parameters and insert
        /// it into the top scope
        /// </summary>
        /// <param name="name"></param>
        /// <param name="type"></param>
        /// <param name="val"></param>
        /// <returns>
        /// The newly created symbol
        /// </returns>
        ISymbol Insert(string name, IStruct type, int val);

        /// <param name="name"></param>
        /// <returns>
        /// Retrieves the type with name from the innermost scope.
        /// </returns>
        ISymbol FindType(string name);

        /// <param name="name"></param>
        /// <returns>
        /// Retrieves the function with name from the innermost scope.
        /// </returns>
        ISymbol FindFunc(string name);

        /// <param name="name"></param>
        /// <returns>
        /// Retrieves the local variable, function parameter or
        /// constant with the given name, from the innermost scope.
        /// </returns>
        ISymbol FindVar(string name);

        /// <summary>
        /// Set the Locals, NArgs, NLocs of the given function to the values
        /// of the current top scope.
        /// </summary>
        /// <param name="function"></param>
        void SetLocals(ISymbol function);
    }
}