using Micro.Implementation;
using Micro.Interfaces;
using NUnit.Framework;

namespace MicroTests.Implementation
{
    [TestFixture]
    public class ParserTests : CompilerTestBase
    {        
        [Test]
        public void TestSimpleProgram()
        {
            sw.WriteLine("function int add <- int x, int y");
            sw.WriteLine("  add = x + y");
            sw.WriteLine("end");
            sw.WriteLine("function int main");
            sw.WriteLine("  int x");
            sw.WriteLine("  x = 1");
            sw.WriteLine("  call put_i <- call add <- x, 2 end end");
            sw.WriteLine("end");
            Parse();
        }

        [Test]
        public void TestIf()
        {
            sw.WriteLine("function int main");
            sw.WriteLine("  int x int y");
            sw.WriteLine("  x = 2 y = 3");
            sw.WriteLine("  if 1 + 2 > x - y + 4");
            sw.WriteLine("    x = y");
            sw.WriteLine("  end");
            sw.WriteLine("end");
            Parse();
        }
        
        [Test]
        public void TestIfElse()
        {
            sw.WriteLine("function int main");
            sw.WriteLine("  int x int y");
            sw.WriteLine("  x = 2 y = 3");
            sw.WriteLine("  if 1 + 2 > x - y + 4");
            sw.WriteLine("    x = y");
            sw.WriteLine("  else");
            sw.WriteLine("    y = x");
            sw.WriteLine("  end");
            sw.WriteLine("end");
            Parse();
        }
        
        public void TestCharLit()
        {
            sw.WriteLine("function int main");
            sw.WriteLine("  char c");
            sw.WriteLine("  c = 'a'");
            sw.WriteLine("end");
            Parse();
        }
        
        [Test]
        public void TestMissingFunction()
        {
            sw.WriteLine("x = 1");
            try {
                Parse();
                Assert.Fail("Function missing, statements must be nested in functions");
            }
            catch (PanicException exc) {
                Assert.AreEqual("Error (1,1): unexpected token, expected: eof, found: ident", exc.Message);
            }
        }

        private void Parse()
        {
            sw.Flush();
            source.Position = 0;
            IError err = new PanicError();
            IScanner s = new Scanner(source, err);
            IParser p = new Parser(s, err, new CodeGen("dummy", err));
            p.Parse();
        }

    }
}