using Micro.Implementation;
using Micro.Interfaces;
using NUnit.Framework;

namespace MicroTests.Implementation
{
    [TestFixture]
    public class ScannerTests : CompilerTestBase
    {                
        [Test]
        public void TestTokens()
        {
            sw.WriteLine("test 123 'x' + - = # < > , if else end call");
            sw.WriteLine("function <-");
            IScanner s = InitScanner();
            Assert.AreEqual(TokenKind.ident, s.Next().Kind);
            Assert.AreEqual(TokenKind.numberLit, s.Next().Kind);
            Assert.AreEqual(TokenKind.charLit, s.Next().Kind);
            Assert.AreEqual(TokenKind.plus, s.Next().Kind);
            Assert.AreEqual(TokenKind.minus, s.Next().Kind);
            Assert.AreEqual(TokenKind.eql, s.Next().Kind);
            Assert.AreEqual(TokenKind.neq, s.Next().Kind);
            Assert.AreEqual(TokenKind.lss, s.Next().Kind);
            Assert.AreEqual(TokenKind.gtr, s.Next().Kind);
            Assert.AreEqual(TokenKind.comma, s.Next().Kind);
            Assert.AreEqual(TokenKind.ifKW, s.Next().Kind);
            Assert.AreEqual(TokenKind.elseKW, s.Next().Kind);
            Assert.AreEqual(TokenKind.endKW, s.Next().Kind);
            Assert.AreEqual(TokenKind.callKW, s.Next().Kind);
            Assert.AreEqual(TokenKind.functionKW, s.Next().Kind);
            Assert.AreEqual(TokenKind.methPar, s.Next().Kind);
            Assert.AreEqual(TokenKind.eof, s.Next().Kind);
            Assert.AreEqual(TokenKind.eof, s.Next().Kind);
        }
        
        [Test]
        public void TestSingleComment()
        {
            sw.WriteLine("{}");
            IScanner s = InitScanner();
            Assert.AreEqual(TokenKind.eof, s.Next().Kind);
        }

        [Test]
        public void TestCommentedFunction()
        {
            sw.WriteLine("{");
            sw.WriteLine("function int main");
            sw.WriteLine("  c = 'a'");
            sw.WriteLine("end");
            sw.WriteLine("}");
            IScanner s = InitScanner();
            Assert.AreEqual(TokenKind.eof, s.Next().Kind);
        }
        
        [Test]
        public void TestNestedComment()
        {
            sw.WriteLine("{");
            sw.WriteLine("function int main");
            sw.WriteLine("  {c = 'a'}");
            sw.WriteLine("end");
            sw.WriteLine("}");
            IScanner s = InitScanner();
            Assert.AreEqual(TokenKind.eof, s.Next().Kind);
        }
        
        [Test]
        public void TestCodeCommentCode()
        {
            sw.WriteLine("function int main");
            sw.WriteLine("  {some funny comment}");
            sw.WriteLine("end");
            IScanner s = InitScanner();
            Assert.AreEqual(TokenKind.functionKW, s.Next().Kind);
            Assert.AreEqual(TokenKind.ident, s.Next().Kind);
            Assert.AreEqual(TokenKind.ident, s.Next().Kind);
            Assert.AreEqual(TokenKind.endKW, s.Next().Kind);
            Assert.AreEqual(TokenKind.eof, s.Next().Kind);
        }
        
        [Test]
        public void TestCommentCodeComment()
        {
            sw.WriteLine("{The main function is the program entry point}");
            sw.WriteLine("function int main");
            sw.WriteLine("  main = 0");
            sw.WriteLine("end");
            sw.WriteLine("{Nothing to be done after here}");
            IScanner s = InitScanner();
            Assert.AreEqual(TokenKind.functionKW, s.Next().Kind);
            Assert.AreEqual(TokenKind.ident, s.Next().Kind);
            Assert.AreEqual(TokenKind.ident, s.Next().Kind);
            Assert.AreEqual(TokenKind.ident, s.Next().Kind);
            Assert.AreEqual(TokenKind.eql, s.Next().Kind);
            Assert.AreEqual(TokenKind.numberLit, s.Next().Kind);
            Assert.AreEqual(TokenKind.endKW, s.Next().Kind);
            Assert.AreEqual(TokenKind.eof, s.Next().Kind);
        }
        
        private IScanner InitScanner()
        {
            sw.Flush();
            source.Position = 0;
            IError err = new PanicError();
            return new Scanner(source, err);
        }

    }
}